Crate rust_tagged

Crate rust_tagged 

Structs

Id
This is just a marker type for macro transformation.
Tagged
rust-tagged provides a simple way to define strongly typed wrappers over primitive types like String, i32, Uuid, chrono::DateTime, etc. It helps eliminate bugs caused by misusing raw primitives for conceptually distinct fields such as UserId, Email, ProductId, and more.

Traits

Taggable
Trait to enforce the use of Tagged types in function signatures. This prevents accidental use of raw primitives and ensures type safety.