Tagged

Struct Tagged 

pub struct Tagged<T, Tag> { /* private fields */ }
rust-tagged provides a simple way to define strongly typed wrappers over primitive types like String, i32, Uuid, chrono::DateTime, etc. It helps eliminate bugs caused by misusing raw primitives for conceptually distinct fields such as UserId, Email, ProductId, and more.

Eliminate accidental mixups between similar types (e.g. OrgId vs UserId) Enforce domain modeling in code via the type system Ergonomic .into() support for primitive conversions

Example - Simple

use tagged_core::{Tagged};

#[derive(Debug)]
struct EmailTag;

type Email = Tagged<String, EmailTag>;

fn main() {
    let email: Email = "test@example.com".into();
    println!("Email inner value: {}", email.value());

    // Convert back to String
    let raw: String = email.into();
    println!("Raw String: {raw}");
}

Implementations

Source

impl<T, Tag> Tagged<T, Tag>

pub fn new(value: T) -> Tagged<T, Tag>

pub fn value(&self) -> &T

Source

impl<T, Tag> Tagged<T, Tag>

Example - Mutation

use tagged_core::Tagged;

#[derive(Debug)]
struct Org;

type OrgName = Tagged<String, Org>;

fn main() {
    let mut name = OrgName::new("Codefonsi".into());

    name.set("New Org Name".into());

    println!("Updated Org Name: {}", name.value());
}
pub fn set(&mut self, new_value: T)

Not allowed feature - Get a mutable reference to the internal value Replace the inner value